#! /usr/bin/env node
"use strict";
/* c8 ignore start */
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const list_files_js_1 = require("../modules/list-files.js");
const get_arg_js_1 = require("../helpers/get-arg.js");
const poku_js_1 = require("../modules/poku.js");
const processes_js_1 = require("../modules/processes.js");
const get_runtime_js_1 = require("../helpers/get-runtime.js");
const format_js_1 = require("../helpers/format.js");
const logs_js_1 = require("../helpers/logs.js");
const hr_js_1 = require("../helpers/hr.js");
(() => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    const dirs = (() => {
        const includeArg = (0, get_arg_js_1.getArg)('include');
        if (includeArg !== undefined)
            return includeArg.split(',');
        const lastParam = (0, get_arg_js_1.getLastParam)();
        if (lastParam !== undefined)
            return lastParam.split(',');
        return ['.'];
    })();
    const platform = (0, get_arg_js_1.getArg)('platform');
    const filter = (0, get_arg_js_1.getArg)('filter');
    const exclude = (0, get_arg_js_1.getArg)('exclude');
    const killPort = (0, get_arg_js_1.getArg)('kill-port');
    const killRange = (0, get_arg_js_1.getArg)('kill-range');
    const killPID = (0, get_arg_js_1.getArg)('kill-pid');
    const denoAllow = (0, get_arg_js_1.argToArray)('deno-allow');
    const denoDeny = (0, get_arg_js_1.argToArray)('deno-deny');
    const denoCJS = ((_a = (0, get_arg_js_1.getArg)('deno-cjs')) === null || _a === void 0 ? void 0 : _a.split(',').map((a) => a.trim()).filter((a) => a)) || (0, get_arg_js_1.hasArg)('deno-cjs');
    const parallel = (0, get_arg_js_1.hasArg)('parallel');
    const quiet = (0, get_arg_js_1.hasArg)('quiet');
    const debug = (0, get_arg_js_1.hasArg)('debug');
    const failFast = (0, get_arg_js_1.hasArg)('fail-fast');
    const concurrency = parallel
        ? Number((0, get_arg_js_1.getArg)('concurrency')) || undefined
        : undefined;
    if (killPort) {
        const ports = killPort.split(',').map(Number);
        yield processes_js_1.kill.port(ports);
    }
    if (killRange) {
        const ranges = killRange.split(',');
        for (const range of ranges) {
            const ports = range.split('-').map(Number);
            const startsAt = ports[0];
            const endsAt = ports[1];
            yield processes_js_1.kill.range(startsAt, endsAt);
        }
    }
    if (killPID) {
        const PIDs = killPID.split(',').map(Number);
        yield processes_js_1.kill.pid(PIDs);
    }
    const options = {
        platform: (0, get_runtime_js_1.platformIsValid)(platform) ? platform : undefined,
        filter: filter ? new RegExp((0, list_files_js_1.escapeRegExp)(filter)) : undefined,
        exclude: exclude ? new RegExp((0, list_files_js_1.escapeRegExp)(exclude)) : undefined,
        parallel,
        quiet,
        debug,
        failFast,
        concurrency,
        deno: {
            allow: denoAllow,
            deny: denoDeny,
            cjs: denoCJS,
        },
    };
    if (debug) {
        (0, hr_js_1.hr)();
        (0, logs_js_1.write)(`${format_js_1.format.bg(104, 'Debug Enabled')}\n`);
        (0, logs_js_1.write)(`${format_js_1.format.italic(format_js_1.format.info('…'))} ${format_js_1.format.bold('Paths')}`);
        console.table(dirs);
        (0, logs_js_1.write)('\n');
        (0, logs_js_1.write)(`${format_js_1.format.italic(format_js_1.format.info('…'))} ${format_js_1.format.bold('Options')}`);
        console.dir(options, { depth: null, colors: true });
    }
    (0, poku_js_1.poku)(dirs, options);
}))();
/* c8 ignore stop */
