"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
var _a;
Object.defineProperty(exports, "__esModule", { value: true });
exports.listFiles = exports.isFile = exports.escapeRegExp = exports.sanitizePath = void 0;
const node_process_1 = __importDefault(require("process"));
const node_fs_1 = require("fs");
const node_path_1 = require("path");
const sanitizePath = (input, ensureTarget) => {
    const sanitizedPath = input
        .replace(/[/\\]+/g, node_path_1.sep) // adapting slashes according to OS
        .replace(/(\.\.(\/|\\|$))+/g, '') // ensure the current path level
        .replace(/[<>|^?*]+/g, ''); // removing unusual path characters
    return ensureTarget ? sanitizedPath.replace(/^[/\\]/, './') : sanitizedPath;
};
exports.sanitizePath = sanitizePath;
const escapeRegExp = (string) => string.replace(/[.*{}[\]\\]/g, '\\$&');
exports.escapeRegExp = escapeRegExp;
const isFile = (fullPath) => {
    return new Promise((resolve, reject) => {
        (0, node_fs_1.stat)(fullPath, (err, stats) => {
            /* c8 ignore next */
            if (err)
                return reject(err);
            resolve(stats.isFile());
        });
    });
};
exports.isFile = isFile;
const envFilter = ((_a = node_process_1.default.env.FILTER) === null || _a === void 0 ? void 0 : _a.trim())
    ? new RegExp((0, exports.escapeRegExp)(node_process_1.default.env.FILTER), 'i')
    : undefined;
const getAllFiles = (dirPath, files = [], configs, callback) => {
    (0, node_fs_1.readdir)((0, exports.sanitizePath)(dirPath), (err, currentFiles) => {
        if (err)
            return callback === null || callback === void 0 ? void 0 : callback(err);
        const defaultRegExp = /\.(test|spec)\./i;
        const filter = (envFilter
            ? envFilter
            : (configs === null || configs === void 0 ? void 0 : configs.filter) instanceof RegExp
                ? configs.filter
                : defaultRegExp) || defaultRegExp;
        const exclude = (configs === null || configs === void 0 ? void 0 : configs.exclude)
            ? Array.isArray(configs.exclude)
                ? /* c8 ignore next */
                    configs.exclude
                : [configs.exclude]
            : undefined;
        let pending = currentFiles.length;
        if (!pending)
            return callback === null || callback === void 0 ? void 0 : callback(null, files);
        currentFiles.forEach((file) => {
            const fullPath = (0, node_path_1.join)(dirPath, file);
            (0, node_fs_1.stat)(fullPath, (err, stat) => {
                /* c8 ignore start */
                if (err) {
                    if (!--pending)
                        callback === null || callback === void 0 ? void 0 : callback(null, files);
                    return;
                }
                /* c8 ignore stop */
                if (fullPath.indexOf('node_modules') !== -1 ||
                    fullPath.indexOf('.git') === 0) {
                    /* c8 ignore start */
                    if (!--pending)
                        callback === null || callback === void 0 ? void 0 : callback(null, files);
                    return;
                    /* c8 ignore stop */
                }
                if (exclude) {
                    for (let i = 0; i < exclude.length; i++) {
                        if (exclude[i].test(fullPath)) {
                            /* c8 ignore start */
                            if (!--pending)
                                callback === null || callback === void 0 ? void 0 : callback(null, files);
                            return;
                            /* c8 ignore stop */
                        }
                    }
                }
                if (filter.test(fullPath)) {
                    files.push(fullPath);
                    /* c8 ignore start */
                    if (!--pending)
                        callback === null || callback === void 0 ? void 0 : callback(null, files);
                    return;
                    /* c8 ignore stop */
                }
                if (stat.isDirectory()) {
                    getAllFiles(fullPath, files, configs, (err) => {
                        /* c8 ignore start */
                        if (err) {
                            if (!--pending)
                                callback === null || callback === void 0 ? void 0 : callback(err, files);
                            return;
                        }
                        /* c8 ignore stop */
                        if (!--pending)
                            callback === null || callback === void 0 ? void 0 : callback(null, files);
                    });
                    /* c8 ignore next */
                }
                else if (!--pending)
                    callback === null || callback === void 0 ? void 0 : callback(null, files);
            });
        });
    });
};
const listFiles = (targetDir, configs) => new Promise((resolve, reject) => {
    getAllFiles((0, exports.sanitizePath)(targetDir), [], configs, (err, result) => {
        if (err)
            return reject(err);
        resolve(result);
    });
});
exports.listFiles = listFiles;
